# Name: PowershellCollector v1.7  
# Author: Mike Measel 
# Chief Counsel and Occasional Muse: Seth Schwartzman
# Created: Feb 14, 2010
#
# Updates
# 2/28/2012 - added cleanup of metrics no longer reporting 
# 3/08/2012 - added parsing to handle xml format metrics
# 3/12/2012 - added error handling for bad metrics
# 3/23/2012 - added parsing to handle xml objects as well as xml values
# 7/01/2012 - changed to allow any script not just powershell
# 11/9/2012 - fixed log4net
# 11/10/2012 - tested against 9.1.1.1 with 9.0.7.1 wilyAgent.dll
# 11/11/2012 - added longaverage and longcounter for really big stuff  
# 11/30/2012 - added [System.GC]::Collect() --  (optional - default on )
#
# Description: Powershell script to create agent connection factories and run powershell scripts and collect output. 

Begin {

# -- log4net stuff
$chome = "C:\Program Files (x86)\CA Wily\PowershellCollector"
    [system.reflection.assembly]::LoadFrom("$chome\bin\log4net.dll") 
    $configFile = new-object System.IO.FileInfo( "$chome\logging.config.xml")   
    [log4net.Config.XmlConfigurator]::Configure( $configFile )
    $logger = [log4net.LogManager]::getLogger("default")
## --
$env:PSModulePath = $env:PSModulePath + ";$chome\modules" 
Import-Module psepaFunctions
# load any snapin's here and they will be available to the scripts
Add-PSSnapin Citrix* -ErrorAction SilentlyContinue

Add-Type -AssemblyName System.Configuration
set-item -path env:com.wily.introscope.agentProfile -value 'C:\Program Files (x86)\CA Wily\PowershellCollector\PowershellCollector.profile'
$config_path="C:\Program Files (x86)\CA Wily\PowershellCollector\PowershellCollector.config"
[System.AppDomain]::CurrentDomain.SetData("APP_CONFIG_FILE", $config_path)

Add-type -path "C:\Program Files (x86)\CA Wily\PowershellCollector\bin\wily.Agent.dll"

set-location "C:\Program Files (x86)\CA Wily\PowershellCollector\scripts"
$runinin = get-location
$message = 'Running scripts from directory ' +$runinin
#write-host $message
$logger.Info($message); 

$activeMetrics = @{};
$mExpireTime = 2                     # how many minutes before expire metric for not reporting

# tell powershell to load our properties file


}

Process {

$keep_going = 1

while ($keep_going -eq 1) {
	
	$scripts = dir
	
	foreach ($_ in $scripts) {
		
		if ( $_.extension -eq ".ps1" ) {
		$script = ".\"+$_.Name
		 $message = 'starting powershell ' + $script
		 $logger.Debug($message); 
		$metrics = Invoke-Expression $script 
		} else {
			$script = "CSCRIPT /nologo "+$_.Name
			$message = 'starting' + $script
			$logger.Debug($message); 
			$metrics = Invoke-Expression $script 
		}
		
		
		if ($metrics) {
		
			foreach ($_ in $metrics) {
				try {  
				$itype = $_.getType()
				if ($itype -eq [System.Xml.XmlDocument]) {
					$mName = $_.Metric.Name
					$mValue = $_.Metric.Value
					$mType = $_.Metric.'Type'
					$logger.debug($mType + " " + $mName + "=" + $mValue)
				} elseif ($_.Contains("<")) {
					[xml]$metricX = $_
					$mName = $metricX.Metric.Name
					$mValue = $metricX.Metric.Value
					$mType = $metricX.Metric.'Type'
					 $logger.debug($mType + " " + $mName + "=" + $mValue)
				} else {
					$metric = $_.split("=")
					$mName = $metric[0]
					$mValue = $metric[1]
					$x2 = 0;
					$isNumber = [System.Int32]::TryParse($mValue, [ref] ($x2))
					if ($isNumber) { $mType = "IntCounter" } else { $mType = "StringEvent" }
					$logger.debug($mType + " " + $mName + "=" + $mValue)
				}
				
				
			$mfName = ($mName |foreach {[regex]::Replace($mName,"[^1-9a-zA-Z_]","")})
			
			$dt  = get-date
			#write-host $mfName
			if ($activeMetrics.ContainsKey("$mfName")) {
				$activeMetrics.Set_Item($mfName, $dt )
				switch ($mType) {
					"IntCounter" {
					$record = '$'+$mfName+".RecordCurrentValue(`"$mValue`")"
					invoke-expression $record }
				        "StringEvent" {
					$record = '$'+$mfName+".RecordDataPoint(`"$mValue`")"
					invoke-expression $record }
					"IntAverage" {
					$record = '$'+$mfName+".RecordDataPoint(`"$mValue`")"
					invoke-expression $record }
			} }
			else {
				switch ($mType) {
					"StringEvent" {
					$metricmill = '[com.wily.introscope.agent.api.StringEventDataRecorder] $'+$mfName+' = [com.wily.introscope.agent.api.DataRecorderFactory]::createStringEventDataRecorder("'+$mName+'")'
					invoke-expression $metricmill
					$record = '$'+$mfName+".RecordDataPoint(`"$mValue`")"
					}
					"IntCounter" {
					$metricmill = '[com.wily.introscope.agent.api.IntCounterDataRecorder] $'+$mfName+' = [com.wily.introscope.agent.api.DataRecorderFactory]::createIntCounterDataRecorder("'+$mName+'")'
					invoke-expression $metricmill
					$record = '$'+$mfName+".RecordCurrentValue(`"$mValue`")"
					}
					"LongCounter" {
					$metricmill = '[com.wily.introscope.agent.api.LongCounterDataRecorder] $'+$mfName+' = [com.wily.introscope.agent.api.DataRecorderFactory]::createLongCounterDataRecorder("'+$mName+'")'
					invoke-expression $metricmill
					$record = '$'+$mfName+".RecordCurrentValue(`"$mValue`")"
					}
					"IntAverage" {
					$metricmill = '[com.wily.introscope.agent.api.IntAverageDataRecorder] $'+$mfName+' = [com.wily.introscope.agent.api.DataRecorderFactory]::createIntAverageDataRecorder("'+$mName+'")'
					invoke-expression $metricmill
					$record = '$'+$mfName+".RecordDataPoint(`"$mValue`")"
					}
					"LongAverage" {
					$metricmill = '[com.wily.introscope.agent.api.LongAverageDataRecorder] $'+$mfName+' = [com.wily.introscope.agent.api.DataRecorderFactory]::createLongAverageDataRecorder("'+$mName+'")'
					invoke-expression $metricmill
					$record = '$'+$mfName+".RecordDataPoint(`"$mValue`")"
				}}
				invoke-expression $record
				$activeMetrics.add($mfName, $dt) 
			}
			
			
			} catch {  write-error "Bad metric data from $script - $_ " }
			}
		#write-host $activeMetrics
		}
		start-sleep 7
		
		# look for stale metrics
		$exptime = [DateTime]::Now.Subtract([TimeSpan]::FromMinutes("$mExpireTime"))
		$deadlist = @()
		foreach ($fname in $activeMetrics.keys) {
			$msl =  ($activeMetrics["$fname"] - $exptime) 
			if ($msl.minutes -lt 1) {
				$deadlist  += "$fname"
				 }
		}
		foreach ($dm in $deadlist) { 
			$activeMetrics.Remove("$dm")
			$reinit = '$'+$dm+".reinitialize()"
			invoke-expression $reinit
			Remove-variable -name "$dm"
			}
	}
		[System.GC]::Collect()

		 #$logger.Debug($plist)  # check global variable value 
		
}	

} 
END { set-location "C:\Program Files (x86)\CA Wily\PowershellCollector" }

